Attribute VB_Name = "modTestSoundex"
' Test module for the Soundex function
' Author: Christian d'Heureuse, chdh@source-code.biz

Option Explicit

Public Const DbServerTypeOracle = 0
Public Const DbServerTypeSqlServer = 1

Private DbServerType  As Integer
Private DbConnection  As ADODB.Connection

' Test routine for the Soundex function.
' Compares the VB Soundex function with the Soundex functions of Oracle
' or Microsoft SQL Server.
Public Sub TestSoundex(ByVal DbServerTypeP As Integer, ByVal DbConnectionString As String)
   DbServerType = DbServerTypeP
   Set DbConnection = New Connection
   DbConnection.Open DbConnectionString
   Dim AllowUnicode As Boolean: AllowUnicode = DbServerType = DbServerTypeSqlServer
   Dim i As Long
   For i = 1 To 1000                         ' (increase count for longer test)
      Dim s As String: s = GenerateRandomString(8, AllowUnicode)
      CompareSoundex s
      Next
   End Sub

' Compares the results of the VB Soundex function and the
' Soundex function of the DB server.
Private Sub CompareSoundex(ByVal s As String)
   Dim s1
   Select Case DbServerType
      Case DbServerTypeOracle:     s1 = Soundex1(s)
      Case DbServerTypeSqlServer:  s1 = Soundex2(s)
      End Select
   Dim s2: s2 = DbServerSoundex(s)
   If IsNull(s1) <> IsNull(s2) Then _
      Err.Raise vbObjectError, , "One Soundex result is Null and the other is not Null."
   If Not IsNull(s1) Then _
      If StrComp(s1, s2, vbBinaryCompare) <> 0 Then _
         Err.Raise vbObjectError, , "The result of the two Soundex functions is not equal!"
   Debug.Print s, s1, s2
   End Sub

' Calls the Soundex function of the DB server via SQL.
Private Function DbServerSoundex(ByVal s As String)
   Dim sql As String
   Select Case DbServerType
      Case DbServerTypeOracle
         sql = "select soundex(" & Quoted(s) & ") as v from dual"
      Case DbServerTypeSqlServer
         sql = "select soundex(" & Quoted(s) & ") as v"
      End Select
   Dim rs As ADODB.Recordset: Set rs = DbConnection.Execute(sql, , adCmdText)
   DbServerSoundex = rs!v.Value
   End Function

' Returns a quoted string. Quotes within the string are duplicated.
Private Function Quoted(ByVal s As String, Optional ByVal Quote As String = "'") As String
   Dim p As Integer: p = 1
   Do
      p = InStr(p, s, Quote, 0)
      If p = 0 Then Exit Do
      s = Left(s, p) & Quote & Mid(s, p + 1)
      p = p + 2
      Loop
   Quoted = Quote & s & Quote
   End Function

' Generates a string with random characters.
Private Function GenerateRandomString(ByVal MaxLength As Integer, ByVal AllowUnicode As Boolean) As String
   Dim Length As Integer: Length = 1 + Int(Rnd * MaxLength)
   Dim s As String: s = ""
   Dim p As Integer
   For p = 1 To Length
      Dim c As Integer
      Dim r: r = Rnd
      If r < 0.8 Then
         c = 65 + Int(Rnd * 58)        ' 80% normal characters
       ElseIf r < 0.9 Then
         c = 32 + Int(Rnd * 96)        ' 10% mixed 32..127
       Else
         If AllowUnicode Then
            c = 32 + Int(Rnd * 1000)   ' 10% Unicode
          Else
            c = 32 + Int(Rnd * 224)    ' 10% 32..255
            End If
         End If
      If AllowUnicode Then
         s = s & ChrW(c)
       Else
         s = s & Chr(c)
         End If
      Next
   GenerateRandomString = s
   End Function
